(** @author: Benoît Montagu <benoit.montagu@inria.fr>
    Copyright © Inria 2022 *)

let ( >>= ) m f = match m with Error _ as err -> err | Ok v -> f v
let return x = Ok x

let run : _ -> unit = function
  | Error msg ->
      Format.printf "%s@." msg;
      exit 2
  | Ok () -> ()

type options = {
  interactive : bool;
  trace : bool;
  tailcalls : bool;
  main : string;
}

let eval options file =
  run
  @@ ( Driver.parse_file Parser.program file >>= fun prog ->
       Check.check prog >>= fun () ->
       Machine.eval ~interactive:options.interactive ~trace:options.trace
         ~tailcalls:options.tailcalls ~main:options.main prog
       >>= fun _v -> return () )
